/**************************************************************************//**
 * @file
 * @brief SPEEX codec demo for EFM32GG
 * @author Energy Micro AS
 * @version 1.04
 ******************************************************************************
 * @section License
 * <b>(C) Copyright 2013 Energy Micro AS, http://www.energymicro.com</b>
 *******************************************************************************
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 * 4. The source and compiled code may only be used on Energy Micro "EFM32"
 *    microcontrollers and "EFR4" radios.
 *
 * DISCLAIMER OF WARRANTY/LIMITATION OF REMEDIES: Energy Micro AS has no
 * obligation to support this Software. Energy Micro AS is providing the
 * Software "AS IS", with no express or implied warranties of any kind,
 * including, but not limited to, any implied warranties of merchantability
 * or fitness for any particular purpose or warranties against infringement
 * of any proprietary rights of a third party.
 *
 * Energy Micro AS will not be liable for any consequential, incidental, or
 * special damages, or any other relief, or for any claim by any third party,
 * arising from your use of this Software.
 *
 *****************************************************************************/
#include <stdlib.h>
#include <stdbool.h>

#include "em_device.h"
#include "em_emu.h"
#include "em_gpio.h"
#include "adc.h"
#include "clock.h"
#include "dac.h"
#include "dma.h"
#include "flash.h"
#include "speexfunc.h"
#include "timer.h"

//#define GG_STK

#ifdef GG_STK
#include "gpio.h"
#else
#include "bsp.h"
#include "voice8k.h"
#endif

struct speexEncFile playList[LIST_SIZE];

/**************************************************************************//**
 * @brief  Main function
 *****************************************************************************/
int main(void)
{
#ifndef GG_STK
  /* Initialize DK board register access, necessary if run on DK */
  BSP_Init(BSP_INIT_DEFAULT);
  BSP_PeripheralAccess(BSP_AUDIO_IN, true);
  BSP_PeripheralAccess(BSP_AUDIO_OUT, true);
#endif
  
  /* Initialize peripherals */
  initClock();
  initDma();
  initAdc();
  initAdcDma();
  initDac();
  initDacDma();
  initFlashDma();
  initTimer();
#ifdef GG_STK
  initGpio();
#endif

  while(1)
  {
/*** Operation for EFM32GG-DK3750 ***/    
#ifndef GG_STK
    /* Wait key press */
    while(!BSP_PushButtonsGet())
      ;
    if (BSP_PushButtonsGet() == BC_UIF_PB1)
    {
      BSP_LedsSet(0x0001);
      /* Wait key release */
      while(BSP_PushButtonsGet()!=0);
      /* Start record */      
      speexRecordFlash512(1, RECORD_FRAMES);
      BSP_LedsSet(0x0000);

      /* Start playback */      
      BSP_LedsSet(0x0100);
      playList[0].frameStart = (char *)FLASHSTART;
      playList[1].frameStart = NULL;
      playList[0].frameNum = RECORD_FRAMES;
      speexPlayBack(1, NARROWBAND8K, FRAME_SIZE_8K, &playList[0]);
      BSP_LedsSet(0x0000);
    }
    if (BSP_PushButtonsGet() == BC_UIF_PB2)
    {    
      BSP_LedsSet(0x0100);
      /* Wait key release */
      while(BSP_PushButtonsGet()!=0);
      /* Start playback */      
      playList[0].frameStart = (char *)FLASHSTART;
      playList[1].frameStart = NULL;
      playList[0].frameNum = RECORD_FRAMES;
      speexPlayBack(1, NARROWBAND8K, FRAME_SIZE_8K, &playList[0]);
      BSP_LedsSet(0x0000);
    }
    if (BSP_PushButtonsGet() == BC_UIF_PB3)
    {    
      BSP_LedsSet(0x0002);
      /* Wait key release */
      while(BSP_PushButtonsGet()!=0);
      /* Start play files */      
      playList[0].frameStart = (char *)voice8k;
      playList[1].frameStart = NULL;
      playList[0].frameNum = sizeof(voice8k)/FRAME_SIZE_8K;
      speexPlayBack(0, NARROWBAND8K, FRAME_SIZE_8K, &playList[0]);
      BSP_LedsSet(0x0000);
    }
    
/*** Operation for EFM32GG-STK3750 ***/        
#else
    /* Wait key press */
    EMU_EnterEM3(true);

    switch (keyCheck())
    {
    case 1:
      GPIO_PinOutSet(gpioPortE, 3);
      /* Start record */      
      speexRecordFlash512(1, RECORD_FRAMES);
      GPIO_PinOutClear(gpioPortE, 3);

      /* Start playback */      
      GPIO_PinOutSet(gpioPortE, 2);
      playList[0].frameStart = (char *)FLASHSTART;
      playList[1].frameStart = NULL;
      playList[0].frameNum = RECORD_FRAMES;
      speexPlayBack(1, NARROWBAND8K, FRAME_SIZE_8K, &playList[0]);
      GPIO_PinOutClear(gpioPortE, 2);
      break;
      
    case 2:
      GPIO_PinOutSet(gpioPortE, 2);
      /* Start playback */      
      playList[0].frameStart = (char *)FLASHSTART;
      playList[1].frameStart = NULL;
      playList[0].frameNum = RECORD_FRAMES;
      speexPlayBack(1, NARROWBAND8K, FRAME_SIZE_8K, &playList[0]);
      GPIO_PinOutClear(gpioPortE, 2);
      break;
      
    default:
      break;
    }
#endif    
  }
}
